/*
 * libfeedback
 * Copyright (c) 2015 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <stdio.h>
#include <stdlib.h>
#include <vconf.h>
#include <system_info.h>

#include "feedback.h"
#include "feedback-internal.h"
#include "profiles.h"
#include "log.h"

/* Temporary keys */
#ifndef VCONFKEY_SETAPPL_VIBRATE_WHEN_NOTIFICATION_BOOL
#define VCONFKEY_SETAPPL_VIBRATE_WHEN_NOTIFICATION_BOOL "db/setting/sound/vibrate_when_notification"
#endif
#ifndef VCONFKEY_SETAPPL_HAPTIC_FEEDBACK_STATUS_BOOL
#define VCONFKEY_SETAPPL_HAPTIC_FEEDBACK_STATUS_BOOL "db/setting/sound/haptic_feedback_on"
#endif


static const char *profile_str_type[] = {
	"FEEDBACK_TYPE_NONE",
	"FEEDBACK_TYPE_SOUND",
	"FEEDBACK_TYPE_VIBRATION",
};

static const struct pattern_field mobile_str_pattern[] = {
	{"FEEDBACK_PATTERN_TAP", FEEDBACK_PATTERN_TAP},
	{"FEEDBACK_PATTERN_SIP", FEEDBACK_PATTERN_SIP},

	{"FEEDBACK_PATTERN_KEY0", FEEDBACK_PATTERN_KEY0},
	{"FEEDBACK_PATTERN_KEY1", FEEDBACK_PATTERN_KEY1},
	{"FEEDBACK_PATTERN_KEY2", FEEDBACK_PATTERN_KEY2},
	{"FEEDBACK_PATTERN_KEY3", FEEDBACK_PATTERN_KEY3},
	{"FEEDBACK_PATTERN_KEY4", FEEDBACK_PATTERN_KEY4},
	{"FEEDBACK_PATTERN_KEY5", FEEDBACK_PATTERN_KEY5},
	{"FEEDBACK_PATTERN_KEY6", FEEDBACK_PATTERN_KEY6},
	{"FEEDBACK_PATTERN_KEY7", FEEDBACK_PATTERN_KEY7},
	{"FEEDBACK_PATTERN_KEY8", FEEDBACK_PATTERN_KEY8},
	{"FEEDBACK_PATTERN_KEY9", FEEDBACK_PATTERN_KEY9},
	{"FEEDBACK_PATTERN_KEY_STAR", FEEDBACK_PATTERN_KEY_STAR},
	{"FEEDBACK_PATTERN_KEY_SHARP", FEEDBACK_PATTERN_KEY_SHARP},
	{"FEEDBACK_PATTERN_KEY_BACK", FEEDBACK_PATTERN_KEY_BACK},
	{"FEEDBACK_PATTERN_HOLD", FEEDBACK_PATTERN_HOLD},

	{"FEEDBACK_PATTERN_HW_TAP", FEEDBACK_PATTERN_HW_TAP},
	{"FEEDBACK_PATTERN_HW_HOLD", FEEDBACK_PATTERN_HW_HOLD},

	{"FEEDBACK_PATTERN_MESSAGE", FEEDBACK_PATTERN_MESSAGE},
	{"FEEDBACK_PATTERN_MESSAGE_ON_CALL", FEEDBACK_PATTERN_MESSAGE_ON_CALL},
	{"FEEDBACK_PATTERN_EMAIL", FEEDBACK_PATTERN_EMAIL},
	{"FEEDBACK_PATTERN_EMAIL_ON_CALL", FEEDBACK_PATTERN_EMAIL_ON_CALL},
	{"FEEDBACK_PATTERN_WAKEUP", FEEDBACK_PATTERN_WAKEUP},
	{"FEEDBACK_PATTERN_WAKEUP_ON_CALL", FEEDBACK_PATTERN_WAKEUP_ON_CALL},
	{"FEEDBACK_PATTERN_SCHEDULE", FEEDBACK_PATTERN_SCHEDULE},
	{"FEEDBACK_PATTERN_SCHEDULE_ON_CALL", FEEDBACK_PATTERN_SCHEDULE_ON_CALL},
	{"FEEDBACK_PATTERN_TIMER", FEEDBACK_PATTERN_TIMER},
	{"FEEDBACK_PATTERN_TIMER_ON_CALL", FEEDBACK_PATTERN_TIMER_ON_CALL},
	{"FEEDBACK_PATTERN_GENERAL", FEEDBACK_PATTERN_GENERAL},
	{"FEEDBACK_PATTERN_GENERAL_ON_CALL", FEEDBACK_PATTERN_GENERAL_ON_CALL},

	{"FEEDBACK_PATTERN_POWERON", FEEDBACK_PATTERN_POWERON},
	{"FEEDBACK_PATTERN_POWEROFF", FEEDBACK_PATTERN_POWEROFF},
	{"FEEDBACK_PATTERN_CHARGERCONN", FEEDBACK_PATTERN_CHARGERCONN},
	{"FEEDBACK_PATTERN_CHARGERCONN_ON_CALL", FEEDBACK_PATTERN_CHARGERCONN_ON_CALL},
	{"FEEDBACK_PATTERN_CHARGING_ERROR", FEEDBACK_PATTERN_CHARGING_ERROR},
	{"FEEDBACK_PATTERN_CHARGING_ERROR_ON_CALL", FEEDBACK_PATTERN_CHARGING_ERROR_ON_CALL},
	{"FEEDBACK_PATTERN_FULLCHARGED", FEEDBACK_PATTERN_FULLCHARGED},
	{"FEEDBACK_PATTERN_FULLCHARGED_ON_CALL", FEEDBACK_PATTERN_FULLCHARGED_ON_CALL},
	{"FEEDBACK_PATTERN_LOWBATT", FEEDBACK_PATTERN_LOWBATT},
	{"FEEDBACK_PATTERN_LOWBATT_ON_CALL", FEEDBACK_PATTERN_LOWBATT_ON_CALL},
	{"FEEDBACK_PATTERN_LOCK", FEEDBACK_PATTERN_LOCK},
	{"FEEDBACK_PATTERN_UNLOCK", FEEDBACK_PATTERN_UNLOCK},

	{"FEEDBACK_PATTERN_OUTGOING_CALL", FEEDBACK_PATTERN_OUTGOING_CALL},

	{"FEEDBACK_PATTERN_VIBRATION_ON", FEEDBACK_PATTERN_VIBRATION_ON},
	{"FEEDBACK_PATTERN_SILENT_OFF", FEEDBACK_PATTERN_SILENT_OFF},
	{"FEEDBACK_PATTERN_BT_CONNECTED", FEEDBACK_PATTERN_BT_CONNECTED},
	{"FEEDBACK_PATTERN_BT_DISCONNECTED", FEEDBACK_PATTERN_BT_DISCONNECTED},

	{"FEEDBACK_PATTERN_SCREEN_CAPTURE", FEEDBACK_PATTERN_SCREEN_CAPTURE},
	{"FEEDBACK_PATTERN_LIST_REORDER", FEEDBACK_PATTERN_LIST_REORDER},
	{"FEEDBACK_PATTERN_LIST_SLIDER", FEEDBACK_PATTERN_LIST_SLIDER},
	{"FEEDBACK_PATTERN_VOLUME_KEY", FEEDBACK_PATTERN_VOLUME_KEY},

	{"FEEDBACK_PATTERN_SEND_SOS_MESSAGE", FEEDBACK_PATTERN_SEND_SOS_MESSAGE},
	{"FEEDBACK_PATTERN_SYSTEM_SHORT", FEEDBACK_PATTERN_SYSTEM_SHORT},
	{"FEEDBACK_PATTERN_SYSTEM_ERROR", FEEDBACK_PATTERN_SYSTEM_ERROR},
	{"FEEDBACK_PATTERN_SYSTEM_LONG", FEEDBACK_PATTERN_SYSTEM_LONG},
	{"FEEDBACK_PATTERN_SYSTEM_MID", FEEDBACK_PATTERN_SYSTEM_MID},
	{"FEEDBACK_PATTERN_RINGER", FEEDBACK_PATTERN_RINGER},
	{"FEEDBACK_PATTERN_NOTIFICATION", FEEDBACK_PATTERN_NOTIFICATION},
};

static const struct pattern_field wearable_str_pattern[] = {
	{"FEEDBACK_PATTERN_TAP", FEEDBACK_PATTERN_TAP},
	{"FEEDBACK_PATTERN_SIP", FEEDBACK_PATTERN_SIP},

	{"FEEDBACK_PATTERN_KEY0", FEEDBACK_PATTERN_KEY0},
	{"FEEDBACK_PATTERN_KEY1", FEEDBACK_PATTERN_KEY1},
	{"FEEDBACK_PATTERN_KEY2", FEEDBACK_PATTERN_KEY2},
	{"FEEDBACK_PATTERN_KEY3", FEEDBACK_PATTERN_KEY3},
	{"FEEDBACK_PATTERN_KEY4", FEEDBACK_PATTERN_KEY4},
	{"FEEDBACK_PATTERN_KEY5", FEEDBACK_PATTERN_KEY5},
	{"FEEDBACK_PATTERN_KEY6", FEEDBACK_PATTERN_KEY6},
	{"FEEDBACK_PATTERN_KEY7", FEEDBACK_PATTERN_KEY7},
	{"FEEDBACK_PATTERN_KEY8", FEEDBACK_PATTERN_KEY8},
	{"FEEDBACK_PATTERN_KEY9", FEEDBACK_PATTERN_KEY9},
	{"FEEDBACK_PATTERN_KEY_STAR", FEEDBACK_PATTERN_KEY_STAR},
	{"FEEDBACK_PATTERN_KEY_SHARP", FEEDBACK_PATTERN_KEY_SHARP},
	{"FEEDBACK_PATTERN_KEY_BACK", FEEDBACK_PATTERN_KEY_BACK},
	{"FEEDBACK_PATTERN_HOLD", FEEDBACK_PATTERN_HOLD},

	{"FEEDBACK_PATTERN_HW_TAP", FEEDBACK_PATTERN_HW_TAP},
	{"FEEDBACK_PATTERN_HW_HOLD", FEEDBACK_PATTERN_HW_HOLD},

	{"FEEDBACK_PATTERN_MESSAGE", FEEDBACK_PATTERN_MESSAGE},
	{"FEEDBACK_PATTERN_MESSAGE_ON_CALL", FEEDBACK_PATTERN_MESSAGE_ON_CALL},
	{"FEEDBACK_PATTERN_EMAIL", FEEDBACK_PATTERN_EMAIL},
	{"FEEDBACK_PATTERN_EMAIL_ON_CALL", FEEDBACK_PATTERN_EMAIL_ON_CALL},
	{"FEEDBACK_PATTERN_WAKEUP", FEEDBACK_PATTERN_WAKEUP},
	{"FEEDBACK_PATTERN_WAKEUP_ON_CALL", FEEDBACK_PATTERN_WAKEUP_ON_CALL},
	{"FEEDBACK_PATTERN_SCHEDULE", FEEDBACK_PATTERN_SCHEDULE},
	{"FEEDBACK_PATTERN_SCHEDULE_ON_CALL", FEEDBACK_PATTERN_SCHEDULE_ON_CALL},
	{"FEEDBACK_PATTERN_TIMER", FEEDBACK_PATTERN_TIMER},
	{"FEEDBACK_PATTERN_TIMER_ON_CALL", FEEDBACK_PATTERN_TIMER_ON_CALL},
	{"FEEDBACK_PATTERN_GENERAL", FEEDBACK_PATTERN_GENERAL},
	{"FEEDBACK_PATTERN_GENERAL_ON_CALL", FEEDBACK_PATTERN_GENERAL_ON_CALL},

	{"FEEDBACK_PATTERN_POWERON", FEEDBACK_PATTERN_POWERON},
	{"FEEDBACK_PATTERN_POWEROFF", FEEDBACK_PATTERN_POWEROFF},
	{"FEEDBACK_PATTERN_CHARGERCONN", FEEDBACK_PATTERN_CHARGERCONN},
	{"FEEDBACK_PATTERN_CHARGERCONN_ON_CALL", FEEDBACK_PATTERN_CHARGERCONN_ON_CALL},
	{"FEEDBACK_PATTERN_CHARGING_ERROR", FEEDBACK_PATTERN_CHARGING_ERROR},
	{"FEEDBACK_PATTERN_CHARGING_ERROR_ON_CALL", FEEDBACK_PATTERN_CHARGING_ERROR_ON_CALL},
	{"FEEDBACK_PATTERN_FULLCHARGED", FEEDBACK_PATTERN_FULLCHARGED},
	{"FEEDBACK_PATTERN_FULLCHARGED_ON_CALL", FEEDBACK_PATTERN_FULLCHARGED_ON_CALL},
	{"FEEDBACK_PATTERN_LOWBATT", FEEDBACK_PATTERN_LOWBATT},
	{"FEEDBACK_PATTERN_LOWBATT_ON_CALL", FEEDBACK_PATTERN_LOWBATT_ON_CALL},
	{"FEEDBACK_PATTERN_LOCK", FEEDBACK_PATTERN_LOCK},
	{"FEEDBACK_PATTERN_UNLOCK", FEEDBACK_PATTERN_UNLOCK},

	{"FEEDBACK_PATTERN_VIBRATION_ON", FEEDBACK_PATTERN_VIBRATION_ON},
	{"FEEDBACK_PATTERN_SILENT_OFF", FEEDBACK_PATTERN_SILENT_OFF},
	{"FEEDBACK_PATTERN_BT_CONNECTED", FEEDBACK_PATTERN_BT_CONNECTED},
	{"FEEDBACK_PATTERN_BT_DISCONNECTED", FEEDBACK_PATTERN_BT_DISCONNECTED},

	{"FEEDBACK_PATTERN_SCREEN_CAPTURE", FEEDBACK_PATTERN_SCREEN_CAPTURE},
	{"FEEDBACK_PATTERN_LIST_REORDER", FEEDBACK_PATTERN_LIST_REORDER},
	{"FEEDBACK_PATTERN_LIST_SLIDER", FEEDBACK_PATTERN_LIST_SLIDER},
	{"FEEDBACK_PATTERN_VOLUME_KEY", FEEDBACK_PATTERN_VOLUME_KEY},

	{"FEEDBACK_PATTERN_SEND_SOS_MESSAGE", FEEDBACK_PATTERN_SEND_SOS_MESSAGE},
	{"FEEDBACK_PATTERN_SYSTEM_SHORT", FEEDBACK_PATTERN_SYSTEM_SHORT},
	{"FEEDBACK_PATTERN_SYSTEM_ERROR", FEEDBACK_PATTERN_SYSTEM_ERROR},
	{"FEEDBACK_PATTERN_SYSTEM_LONG", FEEDBACK_PATTERN_SYSTEM_LONG},
	{"FEEDBACK_PATTERN_SYSTEM_MID", FEEDBACK_PATTERN_SYSTEM_MID},
	{"FEEDBACK_PATTERN_RINGER", FEEDBACK_PATTERN_RINGER},
	{"FEEDBACK_PATTERN_NOTIFICATION", FEEDBACK_PATTERN_NOTIFICATION},
};

static const struct pattern_field common_str_pattern[] = {
	{"FEEDBACK_PATTERN_TAP", FEEDBACK_PATTERN_TAP},
	{"FEEDBACK_PATTERN_SIP", FEEDBACK_PATTERN_SIP},

	{"FEEDBACK_PATTERN_KEY0", FEEDBACK_PATTERN_KEY0},
	{"FEEDBACK_PATTERN_KEY1", FEEDBACK_PATTERN_KEY1},
	{"FEEDBACK_PATTERN_KEY2", FEEDBACK_PATTERN_KEY2},
	{"FEEDBACK_PATTERN_KEY3", FEEDBACK_PATTERN_KEY3},
	{"FEEDBACK_PATTERN_KEY4", FEEDBACK_PATTERN_KEY4},
	{"FEEDBACK_PATTERN_KEY5", FEEDBACK_PATTERN_KEY5},
	{"FEEDBACK_PATTERN_KEY6", FEEDBACK_PATTERN_KEY6},
	{"FEEDBACK_PATTERN_KEY7", FEEDBACK_PATTERN_KEY7},
	{"FEEDBACK_PATTERN_KEY8", FEEDBACK_PATTERN_KEY8},
	{"FEEDBACK_PATTERN_KEY9", FEEDBACK_PATTERN_KEY9},
	{"FEEDBACK_PATTERN_KEY_STAR", FEEDBACK_PATTERN_KEY_STAR},
	{"FEEDBACK_PATTERN_KEY_SHARP", FEEDBACK_PATTERN_KEY_SHARP},
	{"FEEDBACK_PATTERN_KEY_BACK", FEEDBACK_PATTERN_KEY_BACK},
	{"FEEDBACK_PATTERN_HOLD", FEEDBACK_PATTERN_HOLD},

	{"FEEDBACK_PATTERN_HW_TAP", FEEDBACK_PATTERN_HW_TAP},
	{"FEEDBACK_PATTERN_HW_HOLD", FEEDBACK_PATTERN_HW_HOLD},

	{"FEEDBACK_PATTERN_MESSAGE", FEEDBACK_PATTERN_MESSAGE},

	{"FEEDBACK_PATTERN_EMAIL", FEEDBACK_PATTERN_EMAIL},

	{"FEEDBACK_PATTERN_WAKEUP", FEEDBACK_PATTERN_WAKEUP},

	{"FEEDBACK_PATTERN_SCHEDULE", FEEDBACK_PATTERN_SCHEDULE},

	{"FEEDBACK_PATTERN_TIMER", FEEDBACK_PATTERN_TIMER},

	{"FEEDBACK_PATTERN_GENERAL", FEEDBACK_PATTERN_GENERAL},

	{"FEEDBACK_PATTERN_POWERON", FEEDBACK_PATTERN_POWERON},
	{"FEEDBACK_PATTERN_POWEROFF", FEEDBACK_PATTERN_POWEROFF},
	{"FEEDBACK_PATTERN_CHARGERCONN", FEEDBACK_PATTERN_CHARGERCONN},

	{"FEEDBACK_PATTERN_CHARGING_ERROR", FEEDBACK_PATTERN_CHARGING_ERROR},

	{"FEEDBACK_PATTERN_FULLCHARGED", FEEDBACK_PATTERN_FULLCHARGED},

	{"FEEDBACK_PATTERN_LOWBATT", FEEDBACK_PATTERN_LOWBATT},

	{"FEEDBACK_PATTERN_LOCK", FEEDBACK_PATTERN_LOCK},
	{"FEEDBACK_PATTERN_UNLOCK", FEEDBACK_PATTERN_UNLOCK},

	{"FEEDBACK_PATTERN_VIBRATION_ON", FEEDBACK_PATTERN_VIBRATION_ON},
	{"FEEDBACK_PATTERN_SILENT_OFF", FEEDBACK_PATTERN_SILENT_OFF},
	{"FEEDBACK_PATTERN_BT_CONNECTED", FEEDBACK_PATTERN_BT_CONNECTED},
	{"FEEDBACK_PATTERN_BT_DISCONNECTED", FEEDBACK_PATTERN_BT_DISCONNECTED},

	{"FEEDBACK_PATTERN_LIST_REORDER", FEEDBACK_PATTERN_LIST_REORDER},
	{"FEEDBACK_PATTERN_LIST_SLIDER", FEEDBACK_PATTERN_LIST_SLIDER},
	{"FEEDBACK_PATTERN_VOLUME_KEY", FEEDBACK_PATTERN_VOLUME_KEY},
};

typedef enum {
	PROFILE_NONE = -1,
	PROFILE_MOBILE = 1,
	PROFILE_WEARABLE = 2,
	PROFILE_COMMON = 3,
} profile_e;

static int callstatus;
static int lock_sndstatus;
static int camerastatus;
static int shutter_sndstatus;
static int noti_vibstatus;
static int vib_level;
static int noti_level;
static int feedbackstatus;

static char *profile_name = NULL;

#define PROFILE			"tizen.org/feature/profile"
#define PROFILE_MOBILE_STRING	"mobile"
#define PROFILE_WEARABLE_STRING	"wearable"
#define PROFILE_COMMON_STRING	"common"

static profile_e get_profile(void)
{
	int ret;
	int len;
	char *profile = NULL;
	static int set = 0;

	if (set != 0)
		return set;

	ret = system_info_get_platform_string(PROFILE, &profile);
	if (ret < 0) {
		_E("Cannot get profile(%d)", ret);
		return PROFILE_NONE;
	}

	len = strlen(profile) + 1;
	if (!strncmp(PROFILE_MOBILE_STRING, profile, len))
		set = PROFILE_MOBILE;
	else if (!strncmp(PROFILE_WEARABLE_STRING, profile, len))
		set = PROFILE_WEARABLE;
	else
		set = PROFILE_COMMON;

	free(profile);
	return set;
}

static char *profile_get_name()
{
	int prof;

	if (profile_name)
		return profile_name;

	prof = get_profile();

	if (prof == PROFILE_MOBILE)
		profile_name = PROFILE_MOBILE_STRING;
	else if (prof == PROFILE_WEARABLE)
		profile_name = PROFILE_WEARABLE_STRING;
	else
		profile_name = PROFILE_COMMON_STRING;

	return profile_name;
}

/* Return how many patterns profile supports */
static int profile_get_num_of_pattern()
{
	int prof;

	prof = get_profile();

	if (prof == PROFILE_MOBILE)
		return ARR_SIZE(mobile_str_pattern);
	else if (prof == PROFILE_WEARABLE)
		return ARR_SIZE(wearable_str_pattern);
	else
		return ARR_SIZE(common_str_pattern);
}

static const char *profile_str_pattern(int pattern)
{
	int prof;
	int i;

	prof = get_profile();

	if (prof == PROFILE_MOBILE) {
		for (i = 0; i < profile_get_num_of_pattern(); i++) {
			if (pattern == mobile_str_pattern[i].id)
				return mobile_str_pattern[i].string;
		}
	} else if (prof == PROFILE_WEARABLE) {
		for (i = 0; i < profile_get_num_of_pattern(); i++) {
			if (pattern == wearable_str_pattern[i].id)
				return wearable_str_pattern[i].string;
		}
	} else {
		for (i = 0; i < profile_get_num_of_pattern(); i++) {
			if (pattern == common_str_pattern[i].id)
				return common_str_pattern[i].string;
		}
	}
	return "";

}

static int profile_get_pattern_enum(char *pattern)
{
	int prof;
	int i;
	int len;

	prof = get_profile();

	len = strlen(pattern);
	if (prof == PROFILE_MOBILE) {
		for (i = 0; i < profile_get_num_of_pattern(); i++) {
			if (!strncmp(pattern, mobile_str_pattern[i].string, len))
				return mobile_str_pattern[i].id;
		}
	} else if (prof == PROFILE_WEARABLE) {
		for (i = 0; i < profile_get_num_of_pattern(); i++) {
			if (!strncmp(pattern, wearable_str_pattern[i].string, len))
				return wearable_str_pattern[i].id;
		}
	} else {
		for (i = 0; i < profile_get_num_of_pattern(); i++) {
			if (!strncmp(pattern, common_str_pattern[i].string, len))
				return common_str_pattern[i].id;
		}
	}

	return -1;
}

//LCOV_EXCL_START Not called Callback
static void feedback_callstatus_cb(keynode_t *key, void* data)
{
	callstatus = vconf_keynode_get_int(key);
}

static void feedback_lock_sndstatus_cb(keynode_t *key, void* data)
{
	lock_sndstatus = vconf_keynode_get_bool(key);
}

static void feedback_camerastatus_cb(keynode_t *key, void* data)
{
	camerastatus = vconf_keynode_get_int(key);
}

static void feedback_noti_vibstatus_cb(keynode_t *key, void* data)
{
	noti_vibstatus = vconf_keynode_get_bool(key);
}

static void feedback_vib_cb(keynode_t *key, void* data)
{
	vib_level = vconf_keynode_get_int(key);
}

static void feedback_noti_cb(keynode_t *key, void* data)
{
	noti_level = vconf_keynode_get_int(key);
}

static void feedback_feedbackstatus_cb(keynode_t *key, void* data)
{
	feedbackstatus = vconf_keynode_get_bool(key);
}
//LCOV_EXCL_STOP

static void profile_init(void)
{
	int prof;
	int ret;

	prof = get_profile();

	if (prof == PROFILE_COMMON)
		return;
	else if (prof == PROFILE_MOBILE) {
		if (vconf_get_bool(VCONFKEY_SETAPPL_SOUND_LOCK_BOOL, &lock_sndstatus) < 0)
			_W("VCONFKEY_SETAPPL_SOUND_LOCK_BOOL ==> FAIL!!"); //LCOV_EXCL_LINE
		vconf_notify_key_changed(VCONFKEY_SETAPPL_SOUND_LOCK_BOOL, feedback_lock_sndstatus_cb, NULL);
	}

	/* check call status */
	if (vconf_get_int(VCONFKEY_CALL_STATE, &callstatus) < 0)
		_W("VCONFKEY_CALL_STATE ==> FAIL!!"); //LCOV_EXCL_LINE

	/* check camera status */
	if (vconf_get_int(VCONFKEY_CAMERA_STATE, &camerastatus) < 0)
		_W("VCONFKEY_CAMERA_STATE ==> FAIL!!"); //LCOV_EXCL_LINE

	/* shutter sound policy */
	/* This vconf is read just once, because this value is not changed in running time. */
	if (vconf_get_int(VCONFKEY_CAMERA_SHUTTER_SOUND_POLICY, &shutter_sndstatus) < 0)
		_W("VCONFKEY_CAMERA_SHUTTER_SOUND_POLICY ==> FAIL!!"); //LCOV_EXCL_LINE

	/* check vibration status */
	if (vconf_get_bool(VCONFKEY_SETAPPL_VIBRATE_WHEN_NOTIFICATION_BOOL, &noti_vibstatus) < 0)
		_W("VCONFKEY_SETAPPL_VIBRATION_STATUS_BOOL ==> FAIL!!"); //LCOV_EXCL_LINE

	/* check vib_level */
	if (vconf_get_int(VCONFKEY_SETAPPL_TOUCH_FEEDBACK_VIBRATION_LEVEL_INT, &vib_level) < 0)
		_W("VCONFKEY_FEEDBACK_VIBRATION_LEVEL_INT ==> FAIL!!"); //LCOV_EXCL_LINE

	/* check noti_level */
	if (vconf_get_int(VCONFKEY_SETAPPL_NOTI_VIBRATION_LEVEL_INT, &noti_level) < 0)
		_W("VCONFKEY_SETAPPL_NOTI_VIBRATION_LEVEL_INT ==> FAIL!!"); //LCOV_EXCL_LINE

	/* feedback Init */
	if (vconf_get_bool(VCONFKEY_SETAPPL_HAPTIC_FEEDBACK_STATUS_BOOL, &feedbackstatus) < 0)
		_W("VCONFKEY_SETAPPL_HAPTIC_FEEDBACK_STATUS_BOOL ==> FAIL!!"); //LCOV_EXCL_LINE

	/* add watch for status value */
	ret = vconf_notify_key_changed(VCONFKEY_CALL_STATE, feedback_callstatus_cb, NULL);
	if (ret != 0)
		_W("Add watch for VCONFKEY_CALL_STATE failed");
	ret = vconf_notify_key_changed(VCONFKEY_CAMERA_STATE, feedback_camerastatus_cb, NULL);
	if (ret != 0)
		_W("Add watch for VCONFKEY_CAMERA_STATE failed");
	ret = vconf_notify_key_changed(VCONFKEY_SETAPPL_VIBRATE_WHEN_NOTIFICATION_BOOL, feedback_noti_vibstatus_cb, NULL);
	if (ret != 0)
		_W("Add watch for VCONFKEY_SETAPPL_VIBRATE_WHEN_NOTIFICATION_BOOL failed");
	ret = vconf_notify_key_changed(VCONFKEY_SETAPPL_TOUCH_FEEDBACK_VIBRATION_LEVEL_INT, feedback_vib_cb, NULL);
	if (ret != 0)
		_W("Add watch for VCONFKEY_SETAPPL_TOUCH_FEEDBACK_VIBRATION_LEVEL_INT failed");
	ret = vconf_notify_key_changed(VCONFKEY_SETAPPL_NOTI_VIBRATION_LEVEL_INT, feedback_noti_cb, NULL);
	if (ret != 0)
		_W("Add watch for VCONFKEY_SETAPPL_NOTI_VIBRATION_LEVEL_INT failed");
	ret = vconf_notify_key_changed(VCONFKEY_SETAPPL_HAPTIC_FEEDBACK_STATUS_BOOL, feedback_feedbackstatus_cb, NULL);
	if (ret != 0)
		_W("Add watch for VCONFKEY_SETAPPL_HAPTIC_FEEDBACK_STATUS_BOOL failed");
}

static void profile_exit(void)
{
	int prof;
	int ret;

	prof = get_profile();

	if (prof == PROFILE_COMMON)
		return;
	else if (prof == PROFILE_MOBILE)
		vconf_ignore_key_changed(VCONFKEY_SETAPPL_SOUND_LOCK_BOOL, feedback_lock_sndstatus_cb);

	/* remove watch */
	ret = vconf_ignore_key_changed(VCONFKEY_CALL_STATE, feedback_callstatus_cb);
	if (ret != 0)
		_W("Remove watch for VCONFKEY_CALL_STATE failed");
	ret = vconf_ignore_key_changed(VCONFKEY_CAMERA_STATE, feedback_camerastatus_cb);
	if (ret != 0)
		_W("Remove watch for VCONFKEY_CAMERA_STATE failed");
	ret = vconf_ignore_key_changed(VCONFKEY_SETAPPL_VIBRATE_WHEN_NOTIFICATION_BOOL, feedback_noti_vibstatus_cb);
	if (ret != 0)
		_W("Remove watch for VCONFKEY_SETAPPL_VIBRATE_WHEN_NOTIFICATION_BOOL failed");
	ret = vconf_ignore_key_changed(VCONFKEY_SETAPPL_TOUCH_FEEDBACK_VIBRATION_LEVEL_INT, feedback_vib_cb);
	if (ret != 0)
		_W("Remove watch for VCONFKEY_SETAPPL_TOUCH_FEEDBACK_VIBRATION_LEVEL_INT failed");
	ret = vconf_ignore_key_changed(VCONFKEY_SETAPPL_NOTI_VIBRATION_LEVEL_INT, feedback_noti_cb);
	if (ret != 0)
		_W("Remove watch for VCONFKEY_SETAPPL_NOTI_VIBRATION_LEVEL_INT failed");
	ret = vconf_ignore_key_changed(VCONFKEY_SETAPPL_HAPTIC_FEEDBACK_STATUS_BOOL, feedback_feedbackstatus_cb);
	if (ret != 0)
		_W("Remove watch for VCONFKEY_SETAPPL_HAPTIC_FEEDBACK_STATUS_BOOL failed");
}

static bool profile_get_switched_pattern(int pattern, int *switched)
{
	int prof;

	prof = get_profile();

	if (prof == PROFILE_COMMON)
		return false;

	if (!switched)
		return false;

	if (callstatus == VCONFKEY_CALL_OFF)
		return false;

	/* in case of call connected or connecting */
	_D("Call status is connected or connecting."); //LCOV_EXCL_LINE
	if (pattern == FEEDBACK_PATTERN_MESSAGE)
		*switched = FEEDBACK_PATTERN_MESSAGE_ON_CALL;
	else if (pattern == FEEDBACK_PATTERN_EMAIL)
		*switched = FEEDBACK_PATTERN_EMAIL_ON_CALL;
	else if (pattern == FEEDBACK_PATTERN_WAKEUP)
		*switched = FEEDBACK_PATTERN_WAKEUP_ON_CALL;
	else if (pattern == FEEDBACK_PATTERN_SCHEDULE)
		*switched = FEEDBACK_PATTERN_SCHEDULE_ON_CALL;
	else if (pattern == FEEDBACK_PATTERN_TIMER)
		*switched = FEEDBACK_PATTERN_TIMER_ON_CALL;
	else if (pattern == FEEDBACK_PATTERN_GENERAL)
		*switched = FEEDBACK_PATTERN_GENERAL_ON_CALL;
	else if (pattern == FEEDBACK_PATTERN_CHARGERCONN)
		*switched = FEEDBACK_PATTERN_CHARGERCONN_ON_CALL;
	else if (pattern == FEEDBACK_PATTERN_CHARGING_ERROR)
		*switched = FEEDBACK_PATTERN_CHARGING_ERROR_ON_CALL;
	else if (pattern == FEEDBACK_PATTERN_FULLCHARGED)
		*switched = FEEDBACK_PATTERN_FULLCHARGED_ON_CALL;
	else if (pattern == FEEDBACK_PATTERN_LOWBATT)
		*switched = FEEDBACK_PATTERN_LOWBATT_ON_CALL;
	else
		return false;

	return true;
}

static bool mobile_is_notification_type(int pattern)
{
	switch (pattern) {
	case FEEDBACK_PATTERN_WAKEUP:
	case FEEDBACK_PATTERN_WAKEUP_ON_CALL:
	case FEEDBACK_PATTERN_TIMER:
	case FEEDBACK_PATTERN_TIMER_ON_CALL:
	case FEEDBACK_PATTERN_MESSAGE ... FEEDBACK_PATTERN_EMAIL_ON_CALL:
	case FEEDBACK_PATTERN_SCHEDULE:
	case FEEDBACK_PATTERN_SCHEDULE_ON_CALL:
	case FEEDBACK_PATTERN_GENERAL ... FEEDBACK_PATTERN_LOWBATT_ON_CALL:
		return true;
	}
	return false;
}

static bool wearable_is_notification_type(int pattern)
{
	switch (pattern) {
	case FEEDBACK_PATTERN_WAKEUP:
	case FEEDBACK_PATTERN_WAKEUP_ON_CALL:
	case FEEDBACK_PATTERN_TIMER:
	case FEEDBACK_PATTERN_TIMER_ON_CALL:
		return false;
	case FEEDBACK_PATTERN_MESSAGE ... FEEDBACK_PATTERN_EMAIL_ON_CALL:
	case FEEDBACK_PATTERN_SCHEDULE:
	case FEEDBACK_PATTERN_SCHEDULE_ON_CALL:
	case FEEDBACK_PATTERN_GENERAL ... FEEDBACK_PATTERN_LOWBATT_ON_CALL:
		return true;
	}
	return false;
}

static bool profile_is_notification_type(int pattern)
{
	int prof;

	prof = get_profile();

	if (prof == PROFILE_MOBILE)
		return mobile_is_notification_type(pattern);
	else if (prof == PROFILE_WEARABLE)
		return wearable_is_notification_type(pattern);
	else
		return false;
}

static int mobile_get_priority(int pattern)
{
	switch (pattern) {
	case FEEDBACK_PATTERN_TAP ... FEEDBACK_PATTERN_KEY_BACK:
		return PRIORITY_MIN;
	case FEEDBACK_PATTERN_WAKEUP:
	case FEEDBACK_PATTERN_SCHEDULE:
	case FEEDBACK_PATTERN_TIMER:
		return PRIORITY_HIGH;
	case FEEDBACK_PATTERN_SEND_SOS_MESSAGE:
	case FEEDBACK_PATTERN_RINGER:
		return PRIORITY_TOP;
	}

	return PRIORITY_MIDDLE;
}

static int profile_get_priority(int pattern)
{
	int prof;

	prof = get_profile();

	if (prof == PROFILE_MOBILE)
		return mobile_get_priority(pattern);
	else
		return PRIORITY_MIDDLE;
}

static bool mobile_get_always_alert_case(int type, int pattern)
{
	switch (pattern) {
	case FEEDBACK_PATTERN_KEY0 ... FEEDBACK_PATTERN_KEY_BACK:
	case FEEDBACK_PATTERN_HOLD:
		if (CHECK_VIBRATION(type) && feedbackstatus)
			return true;
		break;
	case FEEDBACK_PATTERN_SIP:
		if (CHECK_VIBRATION(type))
			return true;
		break;
	case FEEDBACK_PATTERN_MESSAGE:
	case FEEDBACK_PATTERN_EMAIL:
		if (CHECK_VIBRATION(type) && noti_vibstatus)
			return true;
		break;
	case FEEDBACK_PATTERN_TIMER:
	case FEEDBACK_PATTERN_TIMER_ON_CALL:
		if (CHECK_VIBRATION(type))
			return true;
		break;
	case FEEDBACK_PATTERN_WAKEUP:
	case FEEDBACK_PATTERN_WAKEUP_ON_CALL:
		return true;
	case FEEDBACK_PATTERN_MESSAGE_ON_CALL:
	case FEEDBACK_PATTERN_EMAIL_ON_CALL:
	case FEEDBACK_PATTERN_GENERAL_ON_CALL:
		break;
	case FEEDBACK_PATTERN_CHARGERCONN_ON_CALL:
	case FEEDBACK_PATTERN_CHARGING_ERROR_ON_CALL:
	case FEEDBACK_PATTERN_LOWBATT_ON_CALL:
		/* no matter sound profile */
		if (CHECK_VIBRATION(type))
			return true;
		break;
	case FEEDBACK_PATTERN_SEND_SOS_MESSAGE:
	case FEEDBACK_PATTERN_OUTGOING_CALL:
		return true;
	case FEEDBACK_PATTERN_SCREEN_CAPTURE:
		if (CHECK_SOUND(type) && camerastatus && shutter_sndstatus)
			return true;
		break;
	case FEEDBACK_PATTERN_RINGER:
		return true;
	default:
		break;
	}
	return false;
}

static bool wearable_get_always_alert_case(int type, int pattern)
{
	if (CHECK_SOUND(type)) {
		switch (pattern) {
		case FEEDBACK_PATTERN_WAKEUP:
		case FEEDBACK_PATTERN_WAKEUP_ON_CALL:
			return true;
		case FEEDBACK_PATTERN_MESSAGE_ON_CALL:
		case FEEDBACK_PATTERN_EMAIL_ON_CALL:
		case FEEDBACK_PATTERN_GENERAL_ON_CALL:
			break;
		case FEEDBACK_PATTERN_SEND_SOS_MESSAGE:
			return true;
		case FEEDBACK_PATTERN_SCREEN_CAPTURE:
			if (camerastatus && shutter_sndstatus)
				return true;
			break;
		case FEEDBACK_PATTERN_OUTGOING_CALL:
			return true;
		default:
			break;
		}
	} else if (CHECK_VIBRATION(type)) {
		switch (pattern) {
		case FEEDBACK_PATTERN_TAP:
		case FEEDBACK_PATTERN_KEY0 ... FEEDBACK_PATTERN_KEY_BACK:
		case FEEDBACK_PATTERN_HOLD:
			if (feedbackstatus)
				return true;
			break;
		case FEEDBACK_PATTERN_MESSAGE:
		case FEEDBACK_PATTERN_EMAIL:
		case FEEDBACK_PATTERN_GENERAL:
		case FEEDBACK_PATTERN_SCHEDULE:
			if (noti_vibstatus)
				return true;
			break;
		case FEEDBACK_PATTERN_SIP:
		case FEEDBACK_PATTERN_TIMER:
		case FEEDBACK_PATTERN_TIMER_ON_CALL:
		case FEEDBACK_PATTERN_WAKEUP:
		case FEEDBACK_PATTERN_WAKEUP_ON_CALL:
		case FEEDBACK_PATTERN_MESSAGE_ON_CALL:
		case FEEDBACK_PATTERN_EMAIL_ON_CALL:
		case FEEDBACK_PATTERN_GENERAL_ON_CALL:
		case FEEDBACK_PATTERN_CHARGERCONN_ON_CALL:
		case FEEDBACK_PATTERN_CHARGING_ERROR_ON_CALL:
		case FEEDBACK_PATTERN_LOWBATT_ON_CALL:
		case FEEDBACK_PATTERN_BT_DISCONNECTED:
		case FEEDBACK_PATTERN_SEND_SOS_MESSAGE:
		case FEEDBACK_PATTERN_OUTGOING_CALL:
			return true;
		default:
			break;
		}
	}
	return false;
}

static bool common_get_always_alert_case(int type, int pattern)
{
	switch (pattern) {
	case FEEDBACK_PATTERN_SIP:
	case FEEDBACK_PATTERN_TIMER:
		if (CHECK_VIBRATION(type))
			return true;
		break;
	case FEEDBACK_PATTERN_WAKEUP:
		return true;
	default:
		break;
	}
	return false;
}

static bool profile_get_always_alert_case(int type, int pattern)
{
	int prof;

	prof = get_profile();

	if (prof == PROFILE_MOBILE)
		return mobile_get_always_alert_case(type, pattern);
	else if (prof == PROFILE_WEARABLE)
		return wearable_get_always_alert_case(type, pattern);
	else
		return common_get_always_alert_case(type, pattern);
}

static bool mobile_get_always_off_case(int type, int pattern)
{
	int ret;

	/* check if the state of voice recorder is recording */
	if (vconf_get_int(VCONFKEY_RECORDER_STATE, &ret) < 0) {
		_W("fail to get media sound status, status will be zero"); //LCOV_EXCL_LINE
		ret = 0;
	}

	if (CHECK_SOUND(type) && ret == VCONFKEY_RECORDER_STATE_RECORDING) {
		_D("voice recording status is RECORDING"); //LCOV_EXCL_LINE
		return true;
	}

	switch (pattern) {
	case FEEDBACK_PATTERN_TAP:
	case FEEDBACK_PATTERN_HOLD ... FEEDBACK_PATTERN_HW_HOLD:
		if (CHECK_SOUND(type) && !is_touch_sndstatus())
			return true;
		break;
	case FEEDBACK_PATTERN_KEY0 ... FEEDBACK_PATTERN_KEY_BACK:
		if (CHECK_SOUND(type) && !is_keytone_sndstatus())
			return true;
		else if (CHECK_VIBRATION(type) && !feedbackstatus)
			return true;
		break;
	case FEEDBACK_PATTERN_LOCK:
	case FEEDBACK_PATTERN_UNLOCK:
		if (CHECK_SOUND(type) && !lock_sndstatus)
			return true;
		break;
	default:
		break;
	}
	return false;
}

static bool wearable_is_system_type(int pattern)
{
	switch (pattern) {
	case FEEDBACK_PATTERN_POWERON ... FEEDBACK_PATTERN_VOLUME_KEY:
		return true;
	}
	return false;
}

static bool wearable_get_always_off_case(int type, int pattern)
{
	int ret;

	if (CHECK_SOUND(type)) {
		if ((wearable_is_notification_type(pattern) || wearable_is_system_type(pattern)) &&
		    callstatus) {
			_W("skip %d sound during call", pattern);
			return true;
		}
		/* check if the state of voice recorder is recording */
		if (vconf_get_int(VCONFKEY_RECORDER_STATE, &ret) == 0 &&
		    ret == VCONFKEY_RECORDER_STATE_RECORDING) {
			_D("voice recording status is RECORDING");
			return true;
		}

		switch (pattern) {
		case FEEDBACK_PATTERN_TAP:
		case FEEDBACK_PATTERN_HOLD ... FEEDBACK_PATTERN_HW_HOLD:
			if (!is_touch_sndstatus())
				return true;
			break;
		case FEEDBACK_PATTERN_KEY0 ... FEEDBACK_PATTERN_KEY_BACK:
			if (!is_keytone_sndstatus())
				return true;
			break;
		default:
			break;
		}
	} else if (CHECK_VIBRATION(type)) {
		switch (pattern) {
		case FEEDBACK_PATTERN_TAP:
		case FEEDBACK_PATTERN_KEY0 ... FEEDBACK_PATTERN_KEY_BACK:
		case FEEDBACK_PATTERN_HOLD:
			if (!feedbackstatus)
				return true;
			break;
		default:
			break;
		}
	}
	return false;
}

static bool common_get_always_off_case(int type, int pattern)
{
	switch (pattern) {
	case FEEDBACK_PATTERN_TAP ... FEEDBACK_PATTERN_SIP:
	case FEEDBACK_PATTERN_HOLD ... FEEDBACK_PATTERN_HW_HOLD:
		if (CHECK_SOUND(type) && !is_touch_sndstatus())
			return true;
		break;
	case FEEDBACK_PATTERN_KEY0 ... FEEDBACK_PATTERN_KEY_BACK:
		if (CHECK_SOUND(type) && !is_keytone_sndstatus())
			return true;
		break;
	default:
		break;
	}
	return false;
}

static bool profile_get_always_off_case(int type, int pattern)
{
	int prof;

	prof = get_profile();

	if (prof == PROFILE_MOBILE)
		return mobile_get_always_off_case(type, pattern);
	else if (prof == PROFILE_WEARABLE)
		return wearable_get_always_off_case(type, pattern);
	else
		return common_get_always_off_case(type, pattern);
}

static int mobile_get_strength_type(int type, int pattern)
{
	if (CHECK_SOUND(type)) {
		if (pattern == FEEDBACK_PATTERN_TAP)
			return VOLUME_TYPE_SYSTEM|VOLUME_GAIN_TOUCH;
		else if (pattern >= FEEDBACK_PATTERN_KEY0 && pattern <= FEEDBACK_PATTERN_KEY_BACK)
			return VOLUME_TYPE_SYSTEM|VOLUME_GAIN_DIALER;
		else if (pattern == FEEDBACK_PATTERN_VOLUME_KEY)
			return VOLUME_TYPE_RINGTONE;
		else if (camerastatus && shutter_sndstatus && pattern == FEEDBACK_PATTERN_SCREEN_CAPTURE)
			return VOLUME_TYPE_FIXED;
		else
			return VOLUME_TYPE_SYSTEM;
	} else if (CHECK_VIBRATION(type)) {
		if (pattern >= FEEDBACK_PATTERN_MESSAGE &&
		    pattern <= FEEDBACK_PATTERN_GENERAL)
			return noti_level * HAPTIC_FEEDBACK_STEP;
		else
			return vib_level * HAPTIC_FEEDBACK_STEP;
	}

	return -EINVAL;
}

static int wearable_get_strength_type(int type, int pattern)
{
	if (CHECK_SOUND(type)) {
		if (pattern == FEEDBACK_PATTERN_TAP)
			return VOLUME_TYPE_SYSTEM|VOLUME_GAIN_TOUCH;
		else if (pattern >= FEEDBACK_PATTERN_KEY0 && pattern <= FEEDBACK_PATTERN_KEY_BACK)
			return VOLUME_TYPE_SYSTEM|VOLUME_GAIN_DIALER;
		else if (pattern == FEEDBACK_PATTERN_VOLUME_KEY || pattern == FEEDBACK_PATTERN_SILENT_OFF)
			return VOLUME_TYPE_RINGTONE;
		else if (camerastatus && shutter_sndstatus && pattern == FEEDBACK_PATTERN_SCREEN_CAPTURE)
			return VOLUME_TYPE_FIXED;
		return VOLUME_TYPE_SYSTEM;
	} else if (CHECK_VIBRATION(type)) {
		if (pattern >= FEEDBACK_PATTERN_TAP && pattern <= FEEDBACK_PATTERN_HW_HOLD) {
			_D("use default level");
			return DEFAULT_VIB_LEVEL * HAPTIC_FEEDBACK_STEP;
		}
		/* if call status is ON, vibration magnitude is 20% */
		if (pattern != FEEDBACK_PATTERN_RINGER &&
		    (callstatus == VCONFKEY_CALL_VOICE_ACTIVE || callstatus == VCONFKEY_CALL_VIDEO_ACTIVE)) {
			_W("force change weak level");
			return HAPTIC_FEEDBACK_STEP;
		}
		return noti_level * HAPTIC_FEEDBACK_STEP;
	}
	return -EINVAL;
}

static int common_get_strength_type(int type, int pattern)
{
	if (CHECK_SOUND(type)) {
		if (pattern == FEEDBACK_PATTERN_TAP)
			return VOLUME_TYPE_SYSTEM|VOLUME_GAIN_TOUCH;
		else if (pattern >= FEEDBACK_PATTERN_KEY0 && pattern <= FEEDBACK_PATTERN_KEY_BACK)
			return VOLUME_TYPE_SYSTEM|VOLUME_GAIN_DIALER;
		else if (pattern == FEEDBACK_PATTERN_VOLUME_KEY)
			return VOLUME_TYPE_RINGTONE;
		else
			return VOLUME_TYPE_SYSTEM;
	} else if (CHECK_VIBRATION(type)) {
		return DEFAULT_VIB_LEVEL * HAPTIC_FEEDBACK_STEP;
	}

	return -EINVAL;
}

static int profile_get_strength_type(int type, int pattern)
{
	int prof;

	prof = get_profile();

	if (prof == PROFILE_MOBILE)
		return mobile_get_strength_type(type, pattern);
	else if (prof == PROFILE_WEARABLE)
		return wearable_get_strength_type(type, pattern);
	else
		return common_get_strength_type(type, pattern);
}

static const struct profile_ops feedback_profile_ops = {
	.profile               = profile_get_name,
	.init                  = profile_init,
	.exit                  = profile_exit,
	.get_switched_pattern  = profile_get_switched_pattern,
	.get_always_alert_case = profile_get_always_alert_case,
	.get_always_off_case   = profile_get_always_off_case,
	.get_strength_type     = profile_get_strength_type,
	.get_priority          = profile_get_priority,
	.is_notification_type  = profile_is_notification_type,
	.max_type              = FEEDBACK_TYPE_END,
	.max_pattern           = FEEDBACK_PATTERN_INTERNAL_END,
	.str_type              = profile_str_type,
	.get_num_of_pattern    = profile_get_num_of_pattern,
	.str_pattern           = profile_str_pattern,
	.get_pattern_enum      = profile_get_pattern_enum,
};

PROFILE_OPS_REGISTER(&feedback_profile_ops);
